@@ -1,4 +1,5 @@
-import os, subprocess, shutil, sys, uuid, time, base64
+import os, subprocess, shutil, sys, uuid, time, base64, tempfile
+import requests
 
 from pyspark import SparkConf, SparkContext
 from string import Formatter
@@ -8,7 +9,6 @@ try:
     from kubernetes.client.rest import ApiException
 except ImportError:
     pass
-
 class SparkConfigurationFactory:
 
     def __init__(self, connector):
@@ -77,6 +77,30 @@ class SparkConfiguration(object):
                 _options[name] = value
         return _options
 
+    def fetch_auth_delegation_tokens(self):
+        cluster = self.get_cluster_name()
+        if not cluster or cluster == 'local':
+            return
+        jupyterhub_user_token = os.environ.get('JUPYTERHUB_API_TOKEN')
+        url = os.environ.get('SWAN_HADOOP_TOKEN_GENERATOR_URL')
+        headers = {
+            'Authorization': f'token {jupyterhub_user_token}'
+        }
+        data = {
+            'cluster': cluster
+        }
+        self.connector.log.info(f'Fetching hadoop delegation token for {cluster}')
+        response = requests.post(f'{url}/generate-delegation-token', headers=headers, json=data)
+        response.raise_for_status()
+
+        # Write the token to a temporary file
+        fd, path = tempfile.mkstemp(prefix="hadoop_token_")
+        with os.fdopen(fd, 'wb') as file:
+            file.write(response.content)
+        
+        os.environ['HADOOP_TOKEN_FILE_LOCATION'] = path
+        self.connector.log.info(f'Hadoop delegation token written to {path}')
+
     def configure(self, opts, ports):
         """ Initializes Spark configuration object """
 
@@ -373,11 +397,14 @@ class SparkYarnConfiguration(SparkConfiguration):
                                               '&var-ApplicationId=' + app_id
 
             # determine the history server URL depending on the selected resource manager (yarn, k8s, local etc)
-            history_url = self._get_sc_config(
-                'spark.org.apache.hadoop.yarn.server.webproxy.amfilter.AmIpFilter.param.PROXY_URI_BASES',
+            historyserver_url = self._get_sc_config(
+                'spark.yarn.historyServer.address',
                 wait=True
             )
-            if history_url:
-                conn_config['sparkhistoryserver'] = history_url.split(',', 1)[0]
+            historyserver_protocol = 'https' if self._get_sc_config('spark.ssl.historyServer.enabled') else 'http'
+
+            ui_url = f'{historyserver_protocol}://{historyserver_url}/history/{app_id}'
+            if ui_url:
+                conn_config['sparkhistoryserver'] = ui_url
 
         return conn_config
